package internal

import (
	"context"
	"fmt"
	"github.com/wechatpay-apiv3/wechatpay-go/core"
	"github.com/wechatpay-apiv3/wechatpay-go/core/option"
	"github.com/wechatpay-apiv3/wechatpay-go/services/cashcoupons"
	"github.com/wechatpay-apiv3/wechatpay-go/utils"
	"sync"
)

var once sync.Once
var errorMutex sync.RWMutex

var instanceMap = make(map[string]*cashcoupons.CouponApiService)
var errorMap = make(map[string]error)

func getSrv(ctx context.Context, c *Config) (*cashcoupons.CouponApiService, error) {
	errorMutex.Lock()
	defer errorMutex.Unlock()
	if errorMap[c.MchID] != nil {
		instanceMap[c.MchID] = nil
		errorMap[c.MchID] = nil
	}
	once.Do(func() {
		instanceMap[c.MchID], errorMap[c.MchID] = newInstance(ctx, c)
	})
	return instanceMap[c.MchID], errorMap[c.MchID]
}

func newInstance(ctx context.Context, config *Config) (*cashcoupons.CouponApiService, error) {
	mchPrivateKey, err := utils.LoadPrivateKeyWithPath(config.PrivateKeyPath)
	if err != nil {
		return nil, fmt.Errorf("load merchant private key error:%v", err)
	}
	opts := []core.ClientOption{
		option.WithWechatPayAutoAuthCipher(config.MchID, config.MchCertificateSerialNumber, mchPrivateKey, config.MchAPIv3Key),
	}
	client, err := core.NewClient(ctx, opts...)
	if err != nil {
		return nil, fmt.Errorf("new wechat pay client err:%v", err)
	}
	svc := cashcoupons.CouponApiService{Client: client}
	return &svc, err
}
